@@ -614,18 +614,21 @@ proc restoreSettings {} {
 
 # load modules passed as args designated as requirement
 proc loadRequirementModuleList {tag_list args} {
+   set ret 0
+   set prereqloaded 0
+
    # calling procedure must have already parsed module specification in args
+   set loadedmod_list {}
    foreach mod $args {
-      # get first loaded or loading mod in args list
+      # get all loaded or loading mod in args list
       if {[set loadedmod [getLoadedMatchingName $mod returnfirst]] ne {} ||\
          [set loadedmod [getLoadedMatchingName $mod returnfirst 1]] ne {}} {
-         break
+         lappend loadedmod_list $loadedmod
       }
    }
 
-   if {$loadedmod eq {}} {
+   if {[llength $loadedmod_list] == 0} {
       set imax [llength $args]
-      set prereqloaded 0
       # if prereq list specified, try to load first then
       # try next if load of first module not successful
       for {set i 0} {$i<$imax && $prereqloaded==0} {incr i 1} {
@@ -635,7 +638,8 @@ proc loadRequirementModuleList {tag_list args} {
          # those that failed if one succeed
          set curholdid load-$i-$arg
          lappendState reportholdid $curholdid
-         if {[catch {cmdModuleLoad reqlo 0 $tag_list $arg} errorMsg]} {
+         if {[catch {set retlo [cmdModuleLoad reqlo 0 $tag_list $arg]}\
+            errorMsg]} {
             # if an error is raised, release output and rethrow the error
             # (could be raised if no modulepath defined for instance)
             lpopState reportholdid
@@ -643,6 +647,10 @@ proc loadRequirementModuleList {tag_list args} {
             releaseHeldReport {*}$holdidlist
             knerror $errorMsg
          }
+         # update return value if an issue occurred in cmdModuleLoad
+         if {$retlo != 0} {
+            set ret $retlo
+         }
          lpopState reportholdid
 
          if {[is-loaded $arg]} {
@@ -660,9 +668,12 @@ proc loadRequirementModuleList {tag_list args} {
       # output held messages
       releaseHeldReport {*}$holdidlist
    } else {
-      # apply missing tag to first loaded module found
-      cmdModuleTag 0 0 $tag_list $loadedmod
+      set prereqloaded 1
+      # apply missing tag to all loaded module found
+      cmdModuleTag 0 0 $tag_list {*}$loadedmod_list
    }
+
+   return [list $ret $prereqloaded]
 }
 
 # unload phase of a list of modules reload process
