@@ -1,7 +1,7 @@
 ##########################################################################
 
 # MODSPEC.TCL, module specification procedures
-# Copyright (C) 2016-2021 Xavier Delaruelle
+# Copyright (C) 2016-2022 Xavier Delaruelle
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,9 +28,15 @@ proc defineModStartNbProc {icase} {
    # define proc if not done yet or if it was defined for another context
    if {[info procs modStartNb] eq {} || $::g_modStartNb_proc ne $procname} {
       if {[info exists ::g_modStartNb_proc]} {
+         # remove existing debug trace if any
+         trace remove execution modStartNb enter reportTraceExecEnter
          rename ::modStartNb ::$::g_modStartNb_proc
       }
       rename ::$procname ::modStartNb
+      # set debug trace if verbosity is set to debug2 or higher
+      if {[isVerbosityLevel debug2]} {
+         trace add execution modStartNb enter reportTraceExecEnter
+      }
       set ::g_modStartNb_proc $procname
    }
 }
@@ -93,9 +99,15 @@ proc defineGetEqArrayKeyProc {icase extdfl impdfl} {
    if {[info procs getEqArrayKey] eq {} || $::g_getEqArrayKey_proc ne\
       $procname} {
       if {[info exists ::g_getEqArrayKey_proc]} {
+         # remove existing debug trace if any
+         trace remove execution getEqArrayKey enter reportTraceExecEnter
          rename ::getEqArrayKey ::$::g_getEqArrayKey_proc
       }
       rename ::$procname ::getEqArrayKey
+      # set debug trace if verbosity is set to debug2 or higher
+      if {[isVerbosityLevel debug2]} {
+         trace add execution getEqArrayKey enter reportTraceExecEnter
+      }
       set ::g_getEqArrayKey_proc $procname
    }
 
@@ -213,6 +225,8 @@ proc defineDoesModMatchAtDepthProc {indepth querydepth test} {
    if {[info procs doesModMatchAtDepth] eq {} ||\
       $::g_doesModMatchAtDepth_procprops ne $procprops} {
       if {[info exists ::g_doesModMatchAtDepth_procprops]} {
+         # remove existing debug trace if any
+         trace remove execution doesModMatchAtDepth enter reportTraceExecEnter
          rename ::doesModMatchAtDepth {}
       }
       set ::g_doesModMatchAtDepth_procprops $procprops
@@ -224,6 +238,10 @@ proc defineDoesModMatchAtDepthProc {indepth querydepth test} {
          set atdepth "\[join \[lrange \[split \$mod /\] 0 $querydepth\] /\]"
       }
       proc doesModMatchAtDepth {mod} "return \[modEqStatic $atdepth $test *\]"
+      # set debug trace if verbosity is set to debug2 or higher
+      if {[isVerbosityLevel debug2]} {
+         trace add execution doesModMatchAtDepth enter reportTraceExecEnter
+      }
    }
 }
 
@@ -241,9 +259,15 @@ proc defineModVersCmpProc {icase extdfl} {
    # define proc if not done yet or if it was defined for another context
    if {[info procs modVersCmp] eq {} || $::g_modVersCmp_proc ne $procname} {
       if {[info exists ::g_modVersCmp_proc]} {
+         # remove existing debug trace if any
+         trace remove execution modVersCmp enter reportTraceExecEnter
          rename ::modVersCmp ::$::g_modVersCmp_proc
       }
       rename ::$procname ::modVersCmp
+      # set debug trace if verbosity is set to debug2 or higher
+      if {[isVerbosityLevel debug2]} {
+         trace add execution modVersCmp enter reportTraceExecEnter
+      }
       set ::g_modVersCmp_proc $procname
    }
 }
@@ -466,6 +490,8 @@ proc defineModEqStaticProc {icase extdfl modspec} {
    if {[info procs modEqStatic] eq {} || $::g_modEqStatic_procprops ne\
       $procprops} {
       if {[info exists ::g_modEqStatic_procprops]} {
+         # remove existing debug trace if any
+         trace remove execution modEqStatic enter reportTraceExecEnter
          rename ::modEqStatic {}
       } else {
          # also define modVersCmp which is called by modEqStatic
@@ -543,6 +569,10 @@ proc defineModEqStaticProc {icase extdfl modspec} {
       append procbody "
          return \$ret"
       proc modEqStatic {mod {test equal} {psuf {}}} $procbody
+      # set debug trace if verbosity is set to debug2 or higher
+      if {[isVerbosityLevel debug2]} {
+         trace add execution modEqStatic enter reportTraceExecEnter
+      }
    }
 }
 
@@ -560,9 +590,15 @@ proc defineModEqProc {icase extdfl {loadedmod 0}} {
    # define proc if not done yet or if it was defined for another context
    if {[info procs modEq] eq {} || $::g_modEq_proc ne $procname} {
       if {[info exists ::g_modEq_proc]} {
+         # remove existing debug trace if any
+         trace remove execution modEq enter reportTraceExecEnter
          rename ::modEq ::$::g_modEq_proc
       }
       rename ::$procname ::modEq
+      # set debug trace if verbosity is set to debug2 or higher
+      if {[isVerbosityLevel debug2]} {
+         trace add execution modEq enter reportTraceExecEnter
+      }
       set ::g_modEq_proc $procname
    }
 
@@ -577,7 +613,7 @@ proc defineModEqProc {icase extdfl {loadedmod 0}} {
 
 # alternative definitions of modEq proc
 proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
-   {modvrlist 0}} {
+   {modvrlist 0} {psuf {}}} {
    # extract specified module name from name and version spec
    if {$trspec} {
       lassign [getModuleVersSpec $pattern] pmod pmodname cmpspec versspec\
@@ -585,12 +621,16 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
    } else {
       set pmod $pattern
    }
-   # trim dup trailing / char
+   # trim dup trailing / char and adapt pmod suffix if it starts with /
    if {[string index $pmod end] eq {/}} {
       set pmod [string trimright $pmod /]/
+      set endwslash 1
+   } else {
+      set endwslash 0
    }
    # get alternative names if mod is loading(1) or loaded(2)
    set altlist [switch -- $ismodlo {
+      4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
       2 {getLoadedAltname $mod}
       1 {getAllModuleResolvedName $mod}
@@ -602,6 +642,13 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
    }
    # specified module can be translated in a simple mod name/vers or is empty
    if {$pmod ne {} || $pattern eq {}} {
+      if {$psuf ne {}} {
+         if {$endwslash && [string index $psuf 0] eq {/}} {
+            append pmod [string range $psuf 1 end]
+         } else {
+            append pmod $psuf
+         }
+      }
       if {$test eq {eqstart}} {
          set ret [string equal -length [string length $pmod/] $pmod/ $mod/]
          # apply comparison to alternative names if any and no match for mod
@@ -633,7 +680,7 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
       }
    } elseif {$test eq {eqspec}} {
       # test equality against all version described in spec (list or range
-      # boundaries), trspec is considered enabled
+      # boundaries), trspec is considered enabled and psuf empty
       foreach pmod [getAllModulesFromVersSpec $pattern] {
          if {[set ret [string equal $pmod $mod]]} {
             break
@@ -656,7 +703,7 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
          $pmodname/] $pmodname/ $mod/]} {
          # then compare versions
          set modvers [string range $mod [string length $pmodname/] end]
-         set ret [modVersCmp $cmpspec $versspec $modvers $test]
+         set ret [modVersCmp $cmpspec $versspec $modvers $test $psuf]
       } else {
          set ret 0
       }
@@ -668,7 +715,8 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
                -length [string length $pmodname/] $pmodname/ $alt/]} {
                # then compare versions
                set modvers [string range $alt [string length $pmodname/] end]
-               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test]]} {
+               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test\
+                  $psuf]]} {
                   break
                }
             }
@@ -688,7 +736,7 @@ proc modEqProc {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp 0}\
    return $ret
 }
 proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
-   0} {modvrlist 0}} {
+   0} {modvrlist 0} {psuf {}}} {
    if {$trspec} {
       lassign [getModuleVersSpec $pattern] pmod pmodname cmpspec versspec\
          pmodnamere pmodescglob pmodroot pvrlist
@@ -697,8 +745,12 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
    }
    if {[string index $pmod end] eq {/}} {
       set pmod [string trimright $pmod /]/
+      set endwslash 1
+   } else {
+      set endwslash 0
    }
    set altlist [switch -- $ismodlo {
+      4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
       2 {getLoadedAltname $mod}
       1 {getAllModuleResolvedName $mod}
@@ -708,6 +760,13 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       set mod [getModuleNameAndVersFromVersSpec $mod]
    }
    if {$pmod ne {} || $pattern eq {}} {
+      if {$psuf ne {}} {
+         if {$endwslash && [string index $psuf 0] eq {/}} {
+            append pmod [string range $psuf 1 end]
+         } else {
+            append pmod $psuf
+         }
+      }
       if {$test eq {eqstart}} {
          set ret [string equal -nocase -length [string length $pmod/] $pmod/\
             $mod/]
@@ -738,7 +797,7 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       }
    } elseif {$test eq {eqspec}} {
       # test equality against all version described in spec (list or range
-      # boundaries), trspec is considered enabled
+      # boundaries), trspec is considered enabled and psuf empty
       foreach pmod [getAllModulesFromVersSpec $pattern] {
          if {[set ret [string equal -nocase $pmod $mod]]} {
             break
@@ -761,7 +820,7 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
          -length [string length $pmodname/] $pmodname/ $mod/]} {
          # then compare versions
          set modvers [string range $mod [string length $pmodname/] end]
-         set ret [modVersCmp $cmpspec $versspec $modvers $test]
+         set ret [modVersCmp $cmpspec $versspec $modvers $test $psuf]
       } else {
          set ret 0
       }
@@ -772,7 +831,8 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
                -nocase -length [string length $pmodname/] $pmodname/ $alt/]} {
                # then compare versions
                set modvers [string range $alt [string length $pmodname/] end]
-               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test]]} {
+               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test\
+                  $psuf]]} {
                   break
                }
             }
@@ -790,7 +850,7 @@ proc modEqProcIcase {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
    return $ret
 }
 proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
-   0} {modvrlist 0}} {
+   0} {modvrlist 0} {psuf {}}} {
    if {$trspec} {
       lassign [getModuleVersSpec $pattern] pmod pmodname cmpspec versspec\
          pmodnamere pmodescglob pmodroot pvrlist
@@ -799,8 +859,12 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
    }
    if {[string index $pmod end] eq {/}} {
       set pmod [string trimright $pmod /]/
+      set endwslash 1
+   } else {
+      set endwslash 0
    }
    set altlist [switch -- $ismodlo {
+      4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
       2 {getLoadedAltname $mod}
       1 {getAllModuleResolvedName $mod}
@@ -810,6 +874,13 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       set mod [getModuleNameAndVersFromVersSpec $mod]
    }
    if {$pmod ne {} || $pattern eq {}} {
+      if {$psuf ne {}} {
+         if {$endwslash && [string index $psuf 0] eq {/}} {
+            append pmod [string range $psuf 1 end]
+         } else {
+            append pmod $psuf
+         }
+      }
       if {$test eq {eqstart}} {
          set ret [string equal -length [string length $pmod/] $pmod/ $mod/]
          if {!$ret && [llength $altlist] > 0} {
@@ -856,7 +927,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
       }
    } elseif {$test eq {eqspec}} {
       # test equality against all version described in spec (list or range
-      # boundaries), trspec is considered enabled
+      # boundaries), trspec is considered enabled and psuf empty
       foreach pmod [getAllModulesFromVersSpec $pattern] {
          if {[set ret [string equal $pmod $mod]]} {
             break
@@ -879,7 +950,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
          $pmodname/] $pmodname/ $mod/]} {
          # then compare versions
          set modvers [string range $mod [string length $pmodname/] end]
-         set ret [modVersCmp $cmpspec $versspec $modvers $test]
+         set ret [modVersCmp $cmpspec $versspec $modvers $test $psuf]
       } else {
          set ret 0
       }
@@ -890,7 +961,8 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
                -length [string length $pmodname/] $pmodname/ $alt/]} {
                # then compare versions
                set modvers [string range $alt [string length $pmodname/] end]
-               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test]]} {
+               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test\
+                  $psuf]]} {
                   break
                }
             }
@@ -908,7 +980,7 @@ proc modEqProcExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0} {vrcmp\
    return $ret
 }
 proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
-   {vrcmp 0} {modvrlist 0}} {
+   {vrcmp 0} {modvrlist 0} {psuf {}}} {
    if {$trspec} {
       lassign [getModuleVersSpec $pattern] pmod pmodname cmpspec versspec\
          pmodnamere pmodescglob pmodroot pvrlist
@@ -917,8 +989,12 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
    }
    if {[string index $pmod end] eq {/}} {
       set pmod [string trimright $pmod /]/
+      set endwslash 1
+   } else {
+      set endwslash 0
    }
    set altlist [switch -- $ismodlo {
+      4 {getAllModuleResolvedName $mod 0 {} 1}
       3 {getLoadedAltAndSimplifiedName $mod}
       2 {getLoadedAltname $mod}
       1 {getAllModuleResolvedName $mod}
@@ -928,6 +1004,13 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
       set mod [getModuleNameAndVersFromVersSpec $mod]
    }
    if {$pmod ne {} || $pattern eq {}} {
+      if {$psuf ne {}} {
+         if {$endwslash && [string index $psuf 0] eq {/}} {
+            append pmod [string range $psuf 1 end]
+         } else {
+            append pmod $psuf
+         }
+      }
       if {$test eq {eqstart}} {
          set ret [string equal -nocase -length [string length $pmod/] $pmod/\
             $mod/]
@@ -975,7 +1058,7 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
       }
    } elseif {$test eq {eqspec}} {
       # test equality against all version described in spec (list or range
-      # boundaries), trspec is considered enabled
+      # boundaries), trspec is considered enabled and psuf empty
       foreach pmod [getAllModulesFromVersSpec $pattern] {
          if {[set ret [string equal -nocase $pmod $mod]]} {
             break
@@ -998,7 +1081,7 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
          -length [string length $pmodname/] $pmodname/ $mod/]} {
          # then compare versions
          set modvers [string range $mod [string length $pmodname/] end]
-         set ret [modVersCmp $cmpspec $versspec $modvers $test]
+         set ret [modVersCmp $cmpspec $versspec $modvers $test $psuf]
       } else {
          set ret 0
       }
@@ -1009,7 +1092,8 @@ proc modEqProcIcaseExtdfl {pattern mod {test equal} {trspec 1} {ismodlo 0}\
                -nocase -length [string length $pmodname/] $pmodname/ $alt/]} {
                # then compare versions
                set modvers [string range $alt [string length $pmodname/] end]
-               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test]]} {
+               if {[set ret [modVersCmp $cmpspec $versspec $modvers $test\
+                  $psuf]]} {
                   break
                }
             }
@@ -1098,10 +1182,18 @@ proc defineParseModuleSpecificationProc {advverspec} {
    if {[info procs parseModuleSpecification] eq {} ||\
       $::g_parseModuleSpecification_proc ne $procname} {
       if {[info exists ::g_parseModuleSpecification_proc]} {
+         # remove existing debug trace if any
+         trace remove execution parseModuleSpecification enter\
+            reportTraceExecEnter
          rename ::parseModuleSpecification\
             ::$::g_parseModuleSpecification_proc
       }
       rename ::$procname ::parseModuleSpecification
+      # set debug trace if verbosity is set to debug2 or higher
+      if {[isVerbosityLevel debug2]} {
+         trace add execution parseModuleSpecification enter\
+            reportTraceExecEnter
+      }
       set ::g_parseModuleSpecification_proc $procname
    }
 }
